from json.decoder import JSONDecodeError
from typing import List

from .api import get_access_token, get_organization_detail, send_sms
from .exceptions import GoSmsApiException


def send_batch_sms(client_id: str, client_secret: str, message: str, recipients: List[str]):
    """
    Odešle dávku SMSek na seznam čísel. Použije se první channel v organizaci.
    """

    resp = get_access_token(client_id, client_secret)
    if resp.status_code != 200:
        raise GoSmsApiException(error="can't get access token", status=resp.status_code, reason=resp.content)

    try:
        token = "{} {}".format(resp.json()['token_type'].capitalize(), resp.json()['access_token'])
    except (KeyError, AttributeError, JSONDecodeError):
        raise GoSmsApiException(error="can't get access token", status=400, reason='missing "access token" in response')

    resp = get_organization_detail(token)
    if resp.status_code != 200:
        raise GoSmsApiException(error="can't get organization information", status=resp.status_code, reason=resp.content)

    try:
        channel = resp.json()['channels'][0]['id']
    except (KeyError, JSONDecodeError, IndexError):
        raise GoSmsApiException(error="can't get channel information", status=400, reason='missing "channel" in response')

    send_sms(token, message, recipients, channel)



class GoSmsApiException(Exception):
    status = None
    error = None
    reason = None

    def __init__(self, status=None, error=None, reason=None, *args, **kwargs):
        self.status = status
        self.error = error
        self.reason = reason
        super().__init__(*args, **kwargs)



from typing import List, Optional

import requests
import urllib.parse


BASE_URL = "https://app.gosms.cz"


def get_access_token(client_id: str, client_secret: str) -> requests.Response:
    """
    Vrátí token a platnost

    :return: {
        "access_token":"AccessTokenIU78JO",
        "expires_in":3600,
        "token_type":"bearer",
        "scope":"user"
    }
    """
    url = urllib.parse.urljoin(BASE_URL, "/oauth/v2/token")
    resp = requests.post(url, data={'client_id': client_id,
                                    'client_secret': client_secret,
                                    'grant_type': 'client_credentials'})

    return resp


def get_organization_detail(token: str) -> requests.Response:
    url = urllib.parse.urljoin(BASE_URL, "/api/v1/")
    return requests.get(url, headers={'Authorization': token})


def send_sms(token: str, message: str, recipients: List[str], channel: int, expected_send_start: Optional[str] = None) -> requests.Response:
    """
    Odešle zprávu zadaným recipientům.

    {
        "recipients": {
            "invalid": ["Peter"]
        },
        "link": "api/v1/messages/1"
    }

    """
    url = urllib.parse.urljoin(BASE_URL, "/api/v1/messages/")
    data = {
        'message': message,
        'recipients': recipients,
        'channel': channel,
    }

    if expected_send_start is not None:
        data.update({
            'expected_send_start': expected_send_start
        })

    return requests.post(url, json=data, headers={'Authorization': token})



from setuptools import setup, find_packages


with open('README.md', 'r') as f:
    long_description = f.read()


setup_requires = [
    'wheel',
    'setuptools',
    'setuptools_scm',
]

install_requires = [
    'requests'
]

setup(
    name='gosmscz',
    use_scm_version={
        'write_to': 'gosmscz/version.py'
    },
    author='Creatiweb s.r.o.',
    author_email='vitek@creatiweb.cz',
    description='Go SMS API client library',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://gitlab.com/creatiweb-sro/gosmscz',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    python_requires=">=3.7",
    setup_requires=setup_requires,
    install_requires=install_requires,
    tests_require=[],
)
